@@ -27,7 +27,7 @@ CPAN::Testers::ParseReport - parse reports to www.cpantesters.org from various s
 
 =cut
 
-use version; our $VERSION = qv('0.2.9');
+use version; our $VERSION = qv('0.3.0');
 
 =head1 SYNOPSIS
 
@@ -442,11 +442,11 @@ sub _looks_like_qp {
 
 =head2 $extract = parse_report($target,$dumpvars,%Opt)
 
-Reads one report. $target is the local filename to read. $dumpvars is
-a hashref which gets filled with descriptive stats about
-PASS/FAIL/etc. %Opt are the options as described in the
-C<ctgetreports> manpage. $extract is a hashref containing the found
-variables.
+Reads one report. $target is the local filename to read (but see below
+for option 'article'). $dumpvars is a hashref which gets filled with
+descriptive stats about PASS/FAIL/etc. %Opt are the options as
+described in the C<ctgetreports> manpage. $extract is a hashref
+containing the found variables.
 
 Note: this parsing is a bit dirty but as it seems good enough I'm not
 inclined to change it. We parse HTML with regexps only, not an HTML
@@ -496,6 +496,7 @@ sub parse_report {
     my $expect_prereq = 0;
     my $expect_toolchain = 0;
     my $expecting_toolchain_soon = 0;
+    my $expect_module_versions_report = 0;
     my $fallback_p5 = "";
 
     my $in_summary = 0;
@@ -630,6 +631,8 @@ sub parse_report {
                 if (0) {
                 } elsif (/CPANPLUS, version (\S+)/) {
                     $extract{"meta:writer"} = "CPANPLUS $1";
+		} elsif (/created by (App::cpanminus::reporter \S+)/) {
+		    $extract{"meta:writer"} = $1;
                 } elsif (/created (?:automatically )?by (\S+)/) {
                     $extract{"meta:writer"} = $1;
                     if (/\s+on\s+perl\s+([^,]+),/) {
@@ -695,6 +698,23 @@ sub parse_report {
                     $extract{"meta:output_from"} = $1
                 }
             }
+
+            # Parsing of Module::Versions::Report text in test output
+            if (/Modules in memory:/) {
+                $expect_module_versions_report = 1;
+                next LINE;
+            }
+            elsif ($expect_module_versions_report) {
+                if (/\s+(\S+)(?:\s+(v\d\S+?))?;/) {
+                    $extract{"mod:$1"} = defined $2 ? $2 : 'undef';
+                    next LINE;
+                }
+                elsif (/\[at .+?\]/) {
+                    # trailing timestamp
+                    $expect_module_versions_report = 0;
+                    next LINE;
+                }
+            }
         }
         if ($in_env_context) {
             if ($extract{"meta:writer"} =~ /^CPANPLUS\b/
@@ -731,8 +751,9 @@ sub parse_report {
             if (/^(?:Result:|Files=\d)/) {
                 $in_test_summary = 0;
             } elsif (/^(\S+)\s+\(Wstat:.+?Tests:.+?Failed:\s*(\d+)\)$/) {
-                my $in_test_summary_current_test = $1;
+                my $in_test_summary_current_test = $1; # t/globtest.t or t\globtest.t
                 my $in_test_summary_current_failed = $2;
+                $in_test_summary_current_test =~ s|\\|/|g; # only t/globtest.t
                 $extract{"fail:$in_test_summary_current_test"} = $in_test_summary_current_failed;
             } elsif (/^\s+Failed tests?:/) {
                 # ignoring the exact combination of tests for now, seems like overkill
@@ -871,8 +892,8 @@ sub parse_report {
         if ($extract{"conf:osvers"} && $extract{"conf:archname"}) {
             $extract{"conf:archname+osvers"} = join " ", @extract{"conf:archname","conf:osvers"};
         }
-        if ($extract{"meta:perl"} && $extract{"conf:archname"}) {
-            $extract{"meta:arch+perl"} = join " ", @extract{"conf:archname","meta:perl"};
+        if ($extract{"meta:perl"} && $extract{"conf:osname"}) {
+            $extract{"meta:osname+perl"} = join " ", @extract{"conf:osname","meta:perl"};
         }
         my $data = $dumpvars->{"==DATA=="} ||= [];
         push @$data, \%extract;
@@ -891,7 +912,7 @@ sub parse_report {
         my $have  = $extract{$want} || "";
         $diag .= " $want\[$have]";
     }
-    printf STDERR " %-4s %8d%s\n", $extract{"meta:ok"}, $id, $diag unless $Opt{quiet};
+    printf STDERR " %-4s %8s%s\n", $extract{"meta:ok"}, $id, $diag unless $Opt{quiet};
     if ($Opt{raw}) {
         $report =~ s/\s+\z//;
         print STDERR $report, "\n================\n" unless $Opt{quiet};
@@ -970,8 +991,15 @@ sub _get_cooked_report {
                  ||
                  _looks_like_qp($raw_report)
                 ) {
-            # minimizing MIME effort; don't know about reports in other formats
-            $report = MIME::QuotedPrint::decode_qp($raw_report);
+            # note(1): minimizing MIME effort; don't know about reports in other formats
+            # note(2): Net-Generatus-0.40 had an offending report
+            $report = eval { MIME::QuotedPrint::decode_qp($raw_report) };
+            if (!$report || $@) {
+                warn "WARNING: report '$target' could not be parsed as qp, giving up";
+                if ($raw_report =~ /Subject:.+Dear.+Perl.+Summary/s) {
+                    $report = $raw_report;
+                }
+            }
         } else {
             $report = $raw_report;
         }
